@@ -88,6 +88,13 @@ arg_parser.add_argument(
     default=None,
 )
 
+arg_parser.add_argument(
+    '--enable-faux-immutability',
+    help='Enable faux immutability',
+    action='store_true',
+    default=None,
+)
+
 arg_parser.add_argument(
     '--use-default',
     help='Use default value even if a field is required',
@@ -123,6 +130,14 @@ arg_parser.add_argument(
     default=None,
 )
 
+arg_parser.add_argument(
+    '--use-generic-container-types',
+    help='Use generic container types for type hinting (typing.Sequence, typing.Mapping). '
+    'If `--use-standard-collections` option is set, then import from collections.abc instead of typing',
+    action='store_true',
+    default=None,
+)
+
 arg_parser.add_argument(
     '--use-schema-description',
     help='Use schema description to populate class docstring',
@@ -216,6 +231,19 @@ class Config(BaseModel):
                 )
         return values
 
+    @root_validator
+    def validate_use_generic_container_types(
+        cls, values: Dict[str, Any]
+    ) -> Dict[str, Any]:
+        if values.get('use_generic_container_types'):
+            target_python_version: PythonVersion = values['target_python_version']
+            if target_python_version == target_python_version.PY_36:
+                raise Error(
+                    f"`--use-generic-container-types` can not be used with `--target-python_version` {target_python_version.PY_36.value}.\n"  # type: ignore
+                    " The version will be not supported in a future version"
+                )
+        return values
+
     input: Optional[Path]
     input_file_type: InputFileType = InputFileType.Auto
     output: Optional[Path]
@@ -241,6 +269,8 @@ class Config(BaseModel):
     enum_field_as_literal: Optional[LiteralType] = None
     set_default_enum_member: bool = False
     strict_nullable: bool = False
+    use_generic_container_types: bool = False
+    enable_faux_immutability: bool = False
 
     def merge_args(self, args: Namespace) -> None:
         for field_name in self.__fields__:
@@ -356,6 +386,8 @@ def main(args: Optional[Sequence[str]] = None) -> Exit:
             enum_field_as_literal=config.enum_field_as_literal,
             set_default_enum_member=config.set_default_enum_member,
             strict_nullable=config.strict_nullable,
+            use_generic_container_types=config.use_generic_container_types,
+            enable_faux_immutability=config.enable_faux_immutability,
         )
         return Exit.OK
     except InvalidClassNameError as e:
