@@ -4,15 +4,26 @@ from abc import ABC, abstractmethod
 from collections import defaultdict
 from functools import lru_cache
 from pathlib import Path
-from typing import Any, DefaultDict, Dict, Iterator, List, Optional, Set
+from typing import (
+    Any,
+    ClassVar,
+    DefaultDict,
+    Dict,
+    FrozenSet,
+    Iterator,
+    List,
+    Optional,
+    Set,
+    Tuple,
+)
 
 from jinja2 import Environment, FileSystemLoader, Template
 from pydantic import BaseModel, root_validator
 
 from datamodel_code_generator import cached_property
 from datamodel_code_generator.imports import IMPORT_OPTIONAL, Import
-from datamodel_code_generator.reference import Reference
-from datamodel_code_generator.types import DataType
+from datamodel_code_generator.reference import Reference, _BaseModel
+from datamodel_code_generator.types import DataType, chain_as_tuple
 
 TEMPLATE_DIR: Path = Path(__file__).parents[0] / 'template'
 
@@ -25,7 +36,7 @@ class ConstraintsBase(BaseModel):
     ...
 
 
-class DataModelFieldBase(BaseModel):
+class DataModelFieldBase(_BaseModel):
     name: Optional[str]
     default: Optional[Any]
     required: bool = False
@@ -38,6 +49,15 @@ class DataModelFieldBase(BaseModel):
     constraints: Any = None
     strip_default_none: bool = False
     nullable: Optional[bool] = None
+    parent: Optional[Any] = None
+
+    _exclude_fields: ClassVar[Set[str]] = {'parent'}
+    _pass_fields: ClassVar[Set[str]] = {'parent', 'data_type'}
+
+    def __init__(self, **data: Any):  # type: ignore
+        super().__init__(**data)  # type: ignore
+        if isinstance(self.data_type, DataType) and self.data_type.reference:
+            self.data_type.parent = self
 
     @property
     def type_hint(self) -> str:
@@ -54,16 +74,16 @@ class DataModelFieldBase(BaseModel):
         return f'{OPTIONAL}[{type_hint}]'
 
     @property
-    def imports(self) -> List[Import]:
+    def imports(self) -> Tuple[Import, ...]:
         if self.nullable is None:
             if not self.required:
-                return self.data_type.imports_ + [IMPORT_OPTIONAL]
+                return chain_as_tuple(self.data_type.all_imports, (IMPORT_OPTIONAL,))
         elif self.nullable:
-            return self.data_type.imports_ + [IMPORT_OPTIONAL]
-        return self.data_type.imports_
+            return chain_as_tuple(self.data_type.all_imports, (IMPORT_OPTIONAL,))
+        return tuple(self.data_type.all_imports)
 
     @property
-    def unresolved_types(self) -> Set[str]:
+    def unresolved_types(self) -> FrozenSet[str]:
         return self.data_type.unresolved_types
 
     @property
@@ -130,20 +150,17 @@ class DataModel(TemplateBase, ABC):
 
     def __init__(
         self,
-        name: str,
+        *,
+        reference: Reference,
         fields: List[DataModelFieldBase],
         decorators: Optional[List[str]] = None,
-        base_classes: Optional[List[str]] = None,
+        base_classes: Optional[List[Reference]] = None,
         custom_base_class: Optional[str] = None,
         custom_template_dir: Optional[Path] = None,
         extra_template_data: Optional[DefaultDict[str, Dict[str, Any]]] = None,
-        imports: Optional[List[Import]] = None,
-        auto_import: bool = True,
-        reference_classes: Optional[List[str]] = None,
         methods: Optional[List[str]] = None,
         path: Optional[Path] = None,
         description: Optional[str] = None,
-        reference: Optional[Reference] = None,
     ) -> None:
         if not self.TEMPLATE_FILE_PATH:
             raise Exception('TEMPLATE_FILE_PATH is undefined')
@@ -154,75 +171,88 @@ class DataModel(TemplateBase, ABC):
             if custom_template_file_path.exists():
                 template_file_path = custom_template_file_path
 
-        self.name: str = name
         self.fields: List[DataModelFieldBase] = fields or []
         self.decorators: List[str] = decorators or []
-        self.imports: List[Import] = imports or []
-        self.base_class: Optional[str] = None
-        base_classes = [base_class for base_class in base_classes or [] if base_class]
-        self.base_classes: List[str] = base_classes
-        self.path: Optional[Path] = path
-        self.reference: Optional[Reference] = reference
+        self._additional_imports: List[Import] = []
+        self.base_classes: List[Reference] = [
+            base_class for base_class in base_classes or [] if base_class
+        ]
+        self.custom_base_class = custom_base_class
+        self.file_path: Optional[Path] = path
+        self.reference: Reference = reference
 
-        if self.reference:
-            self.reference.source = self
-
-        self.reference_classes: Set[str] = {
-            r for r in base_classes if r != self.BASE_CLASS
-        } if base_classes else set()
-        if reference_classes:
-            self.reference_classes.update(reference_classes)
-
-        if self.base_classes:
-            self.base_class = ', '.join(self.base_classes)
-        else:
-            base_class_full_path = custom_base_class or self.BASE_CLASS
-            if auto_import:
-                if base_class_full_path:
-                    self.imports.append(Import.from_full_path(base_class_full_path))
-            self.base_class = base_class_full_path.rsplit('.', 1)[-1]
-
-        if '.' in name:
-            module, class_name = name.rsplit('.', 1)
-            prefix = f'{module}.'
-            if self.base_class.startswith(prefix):
-                self.base_class = self.base_class.replace(prefix, '', 1)
-        else:
-            class_name = name
-
-        self.class_name: str = class_name
+        self.reference.source = self
 
         self.extra_template_data = (
             extra_template_data[self.name]
             if extra_template_data is not None
             else defaultdict(dict)
         )
+
+        if not self.base_classes:
+            base_class_full_path = custom_base_class or self.BASE_CLASS
+            if base_class_full_path:
+                self._additional_imports.append(
+                    Import.from_full_path(base_class_full_path)
+                )
+
         if extra_template_data:
             all_model_extra_template_data = extra_template_data.get(ALL_MODEL)
             if all_model_extra_template_data:
                 self.extra_template_data.update(all_model_extra_template_data)
 
-        unresolved_types: Set[str] = {*()}
-        for field in self.fields:
-            unresolved_types.update(field.unresolved_types)
-
-        self.reference_classes |= unresolved_types
-
-        if auto_import:
-            for field in self.fields:
-                self.imports.extend(field.imports)
-
         self.methods: List[str] = methods or []
 
         self.description = description
+        for field in self.fields:
+            field.parent = self
+
         super().__init__(template_file_path=template_file_path)
 
-    @cached_property
+    @property
+    def imports(self) -> Tuple[Import, ...]:
+        return chain_as_tuple(
+            (i for f in self.fields for i in f.imports), self._additional_imports
+        )
+
+    @property
+    def reference_classes(self) -> FrozenSet[str]:
+        return frozenset(
+            {r.path for r in self.base_classes if r.name != self.BASE_CLASS}
+            | {t for f in self.fields for t in f.unresolved_types}
+        )
+
+    @property
+    def name(self) -> str:
+        return self.reference.name
+
+    @property
+    def base_class(self) -> str:
+        if self.base_classes:
+            base_class: str = ', '.join(b.name for b in self.base_classes)
+        else:
+            base_class_full_path = self.custom_base_class or self.BASE_CLASS
+            base_class = base_class_full_path.rsplit('.', 1)[-1]
+
+        if '.' in self.name:
+            module, class_name = self.name.rsplit('.', 1)
+            prefix = f'{module}.'
+            if base_class.startswith(prefix):
+                base_class = base_class.replace(prefix, '', 1)
+        return base_class
+
+    @property
+    def class_name(self) -> str:
+        if '.' in self.name:
+            return self.name.rsplit('.', 1)[-1]
+        return self.name
+
+    @property
     def module_path(self) -> List[str]:
-        if self.path:
+        if self.file_path:
             return [
-                *self.path.parts[:-1],
-                self.path.stem,
+                *self.file_path.parts[:-1],
+                self.file_path.stem,
                 *self.name.split('.')[:-1],
             ]
         return self.name.split('.')[:-1]
@@ -232,6 +262,10 @@ class DataModel(TemplateBase, ABC):
         for field in self.fields:
             yield from field.data_type.all_data_types
 
+    @cached_property
+    def path(self) -> str:
+        return self.reference.path
+
     def render(self) -> str:
         response = self._render(
             class_name=self.class_name,
