@@ -3,6 +3,7 @@ from typing import Any, DefaultDict, List, Optional
 
 from datamodel_code_generator.imports import Import
 from datamodel_code_generator.model import DataModel, DataModelFieldBase
+from datamodel_code_generator.reference import Reference
 
 
 class DataClass(DataModel):
@@ -10,30 +11,29 @@ class DataClass(DataModel):
 
     def __init__(
         self,
-        name: str,
+        *,
+        reference: Reference,
         fields: List[DataModelFieldBase],
         decorators: Optional[List[str]] = None,
-        base_classes: Optional[List[str]] = None,
+        base_classes: Optional[List[Reference]] = None,
         custom_base_class: Optional[str] = None,
         custom_template_dir: Optional[Path] = None,
         extra_template_data: Optional[DefaultDict[str, Any]] = None,
-        auto_import: bool = True,
-        reference_classes: Optional[List[str]] = None,
         path: Optional[Path] = None,
         description: Optional[str] = None,
     ):
 
         super().__init__(
-            name,
-            fields,
-            decorators,
-            base_classes,
+            reference=reference,
+            fields=fields,
+            decorators=decorators,
+            base_classes=base_classes,
             custom_base_class=custom_base_class,
             custom_template_dir=custom_template_dir,
             extra_template_data=extra_template_data,
-            auto_import=auto_import,
-            reference_classes=reference_classes,
             path=path,
             description=description,
         )
-        self.imports.append(Import.from_full_path('pydantic.dataclasses.dataclass'))
+        self._additional_imports.append(
+            Import.from_full_path('pydantic.dataclasses.dataclass')
+        )
