@@ -46,61 +46,61 @@ type_map: Dict[Types, DataType] = {
     Types.number: DataType(type='float'),
     Types.float: DataType(type='float'),
     Types.double: DataType(type='float'),
-    Types.decimal: DataType(type='Decimal', imports_=[IMPORT_DECIMAL]),
+    Types.decimal: DataType(type='Decimal', imports=[IMPORT_DECIMAL]),
     Types.time: DataType(type='time'),
     Types.string: DataType(type='str'),
     Types.byte: DataType(type='str'),  # base64 encoded string
     Types.binary: DataType(type='bytes'),
-    Types.date: DataType(type='date', imports_=[IMPORT_DATE]),
-    Types.date_time: DataType(type='datetime', imports_=[IMPORT_DATETIME]),
-    Types.password: DataType(type='SecretStr', imports_=[IMPORT_SECRET_STR]),
-    Types.email: DataType(type='EmailStr', imports_=[IMPORT_EMAIL_STR]),
-    Types.uuid: DataType(type='UUID', imports_=[IMPORT_UUID]),
-    Types.uuid1: DataType(type='UUID1', imports_=[IMPORT_UUID1]),
-    Types.uuid2: DataType(type='UUID2', imports_=[IMPORT_UUID2]),
-    Types.uuid3: DataType(type='UUID3', imports_=[IMPORT_UUID3]),
-    Types.uuid4: DataType(type='UUID4', imports_=[IMPORT_UUID4]),
-    Types.uuid5: DataType(type='UUID5', imports_=[IMPORT_UUID5]),
-    Types.uri: DataType(type='AnyUrl', imports_=[IMPORT_ANYURL]),
+    Types.date: DataType(type='date', imports=[IMPORT_DATE]),
+    Types.date_time: DataType(type='datetime', imports=[IMPORT_DATETIME]),
+    Types.password: DataType(type='SecretStr', imports=[IMPORT_SECRET_STR]),
+    Types.email: DataType(type='EmailStr', imports=[IMPORT_EMAIL_STR]),
+    Types.uuid: DataType(type='UUID', imports=[IMPORT_UUID]),
+    Types.uuid1: DataType(type='UUID1', imports=[IMPORT_UUID1]),
+    Types.uuid2: DataType(type='UUID2', imports=[IMPORT_UUID2]),
+    Types.uuid3: DataType(type='UUID3', imports=[IMPORT_UUID3]),
+    Types.uuid4: DataType(type='UUID4', imports=[IMPORT_UUID4]),
+    Types.uuid5: DataType(type='UUID5', imports=[IMPORT_UUID5]),
+    Types.uri: DataType(type='AnyUrl', imports=[IMPORT_ANYURL]),
     Types.hostname: DataType(
         type='constr',
-        imports_=[IMPORT_CONSTR],
+        imports=[IMPORT_CONSTR],
         is_func=True,
         # https://github.com/horejsek/python-fastjsonschema/blob/61c6997a8348b8df9b22e029ca2ba35ef441fbb8/fastjsonschema/draft04.py#L31
         kwargs={
             'regex': r"r'^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]{0,61}[A-Za-z0-9])\Z'"
         },
     ),
-    Types.ipv4: DataType(type='IPv4Address', imports_=[IMPORT_IPV4ADDRESS]),
-    Types.ipv6: DataType(type='IPv6Address', imports_=[IMPORT_IPV6ADDRESS]),
+    Types.ipv4: DataType(type='IPv4Address', imports=[IMPORT_IPV4ADDRESS]),
+    Types.ipv6: DataType(type='IPv6Address', imports=[IMPORT_IPV6ADDRESS]),
     Types.boolean: DataType(type='bool'),
-    Types.object: DataType(type='Dict[str, Any]', imports_=[IMPORT_ANY, IMPORT_DICT,],),
-    Types.null: DataType(type='Any', imports_=[IMPORT_ANY], is_optional=True),
-    Types.array: DataType(type='List[Any]', imports_=[IMPORT_LIST, IMPORT_ANY]),
-    Types.any: DataType(type='Any', imports_=[IMPORT_ANY]),
+    Types.object: DataType(type='Dict[str, Any]', imports=[IMPORT_ANY, IMPORT_DICT,],),
+    Types.null: DataType(type='Any', imports=[IMPORT_ANY], is_optional=True),
+    Types.array: DataType(type='List[Any]', imports=[IMPORT_LIST, IMPORT_ANY]),
+    Types.any: DataType(type='Any', imports=[IMPORT_ANY]),
 }
 
 standard_collections_type_map = {
     **type_map,
-    Types.object: DataType(type='dict[str, Any]', imports_=[IMPORT_ANY,],),
-    Types.array: DataType(type='list[Any]', imports_=[IMPORT_ANY]),
+    Types.object: DataType(type='dict[str, Any]', imports=[IMPORT_ANY,],),
+    Types.array: DataType(type='list[Any]', imports=[IMPORT_ANY]),
 }
 
 generic_container_type_map = {
     **type_map,
     Types.object: DataType(
-        type='Mapping[str, Any]', imports_=[IMPORT_MAPPING, IMPORT_ANY],
+        type='Mapping[str, Any]', imports=[IMPORT_MAPPING, IMPORT_ANY],
     ),
-    Types.array: DataType(type='Sequence[Any]', imports_=[IMPORT_SEQUENCE, IMPORT_ANY]),
+    Types.array: DataType(type='Sequence[Any]', imports=[IMPORT_SEQUENCE, IMPORT_ANY]),
 }
 
 generic_container_standard_collections_type_map = {
     **type_map,
     Types.object: DataType(
-        type='Mapping[str, Any]', imports_=[IMPORT_ABC_MAPPING, IMPORT_ANY],
+        type='Mapping[str, Any]', imports=[IMPORT_ABC_MAPPING, IMPORT_ANY],
     ),
     Types.array: DataType(
-        type='Sequence[Any]', imports_=[IMPORT_ABC_SEQUENCE, IMPORT_ANY]
+        type='Sequence[Any]', imports=[IMPORT_ABC_SEQUENCE, IMPORT_ANY]
     ),
 }
 
@@ -163,18 +163,14 @@ class DataTypeManager(_DataTypeManager):
         data_type_kwargs = transform_kwargs(kwargs, number_kwargs)
         if data_type_kwargs:
             if data_type_kwargs == {'gt': 0}:
-                return self.data_type(
-                    type='PositiveInt', imports_=[IMPORT_POSITIVE_INT]
-                )
+                return self.data_type(type='PositiveInt', imports=[IMPORT_POSITIVE_INT])
             if data_type_kwargs == {'lt': 0}:
-                return self.data_type(
-                    type='NegativeInt', imports_=[IMPORT_NEGATIVE_INT]
-                )
+                return self.data_type(type='NegativeInt', imports=[IMPORT_NEGATIVE_INT])
             return self.data_type(
                 type='conint',
                 is_func=True,
                 kwargs={k: int(v) for k, v in data_type_kwargs.items()},
-                imports_=[IMPORT_CONINT],
+                imports=[IMPORT_CONINT],
             )
         return self.type_map[types]
 
@@ -183,17 +179,17 @@ class DataTypeManager(_DataTypeManager):
         if data_type_kwargs:
             if data_type_kwargs == {'gt': 0}:
                 return self.data_type(
-                    type='PositiveFloat', imports_=[IMPORT_POSITIVE_FLOAT]
+                    type='PositiveFloat', imports=[IMPORT_POSITIVE_FLOAT]
                 )
             if data_type_kwargs == {'lt': 0}:
                 return self.data_type(
-                    type='NegativeFloat', imports_=[IMPORT_NEGATIVE_FLOAT]
+                    type='NegativeFloat', imports=[IMPORT_NEGATIVE_FLOAT]
                 )
             return self.data_type(
                 type='confloat',
                 is_func=True,
                 kwargs={k: float(v) for k, v in data_type_kwargs.items()},
-                imports_=[IMPORT_CONFLOAT],
+                imports=[IMPORT_CONFLOAT],
             )
         return self.type_map[types]
 
@@ -204,7 +200,7 @@ class DataTypeManager(_DataTypeManager):
                 type='condecimal',
                 is_func=True,
                 kwargs={k: Decimal(v) for k, v in data_type_kwargs.items()},
-                imports_=[IMPORT_CONDECIMAL],
+                imports=[IMPORT_CONDECIMAL],
             )
         return self.type_map[types]
 
@@ -217,7 +213,7 @@ class DataTypeManager(_DataTypeManager):
                 type='constr',
                 is_func=True,
                 kwargs=data_type_kwargs,
-                imports_=[IMPORT_CONSTR],
+                imports=[IMPORT_CONSTR],
             )
         return self.type_map[types]
 
