@@ -9,6 +9,7 @@ from typing import (
     Callable,
     DefaultDict,
     Dict,
+    Iterable,
     Iterator,
     List,
     Mapping,
@@ -28,8 +29,7 @@ from ..imports import IMPORT_ANNOTATIONS, Import, Imports
 from ..model import pydantic as pydantic_model
 from ..model.base import ALL_MODEL, DataModel, DataModelFieldBase
 from ..model.enum import Enum
-from ..model.pydantic.imports import IMPORT_FIELD
-from ..reference import ModelResolver
+from ..reference import ModelResolver, Reference
 from ..types import DataType, DataTypeManager
 from . import LiteralType
 
@@ -48,7 +48,7 @@ def to_hashable(item: Any) -> Any:
         return tuple(to_hashable(i) for i in item)
     elif isinstance(item, dict):
         return tuple(sorted((k, to_hashable(v),) for k, v in item.items()))
-    elif isinstance(item, set):
+    elif isinstance(item, set):  # pragma: no cover
         return frozenset(to_hashable(i) for i in item)
     elif isinstance(item, BaseModel):
         return to_hashable(item.dict())
@@ -62,8 +62,6 @@ def snakify_field(field: DataModelFieldBase, model: DataModel) -> None:
     field.name = camel_to_snake(original_name)
     if field.name != original_name:
         field.alias = original_name
-        if IMPORT_FIELD not in model.fields:  # pragma: no cover
-            model.imports.append(IMPORT_FIELD)
 
 
 def set_strip_default_none(field: DataModelFieldBase, _: DataModel) -> None:
@@ -94,18 +92,18 @@ def sort_data_models(
     unresolved_references: List[DataModel] = []
     for model in unsorted_data_models:
         if not model.reference_classes:
-            sorted_data_models[model.name] = model
+            sorted_data_models[model.path] = model
         elif (
-            model.name in model.reference_classes and len(model.reference_classes) == 1
+            model.path in model.reference_classes and len(model.reference_classes) == 1
         ):  # only self-referencing
-            sorted_data_models[model.name] = model
-            require_update_action_models.append(model.name)
+            sorted_data_models[model.path] = model
+            require_update_action_models.append(model.path)
         elif (
-            not model.reference_classes - {model.name} - set(sorted_data_models)
+            not model.reference_classes - {model.path} - set(sorted_data_models)
         ):  # reference classes have been resolved
-            sorted_data_models[model.name] = model
-            if model.name in model.reference_classes:
-                require_update_action_models.append(model.name)
+            sorted_data_models[model.path] = model
+            if model.path in model.reference_classes:
+                require_update_action_models.append(model.path)
         else:
             unresolved_references.append(model)
     if unresolved_references:
@@ -123,12 +121,12 @@ def sort_data_models(
         # sort on base_class dependency
         while True:
             ordered_models: List[Tuple[int, DataModel]] = []
-            unresolved_reference_model_names = [m.name for m in unresolved_references]
+            unresolved_reference_model_names = [m.path for m in unresolved_references]
             for model in unresolved_references:
                 indexes = [
-                    unresolved_reference_model_names.index(b)
+                    unresolved_reference_model_names.index(b.path)
                     for b in model.base_classes
-                    if b in unresolved_reference_model_names
+                    if b.path in unresolved_reference_model_names
                 ]
                 if indexes:
                     ordered_models.append((min(indexes), model,))
@@ -145,14 +143,14 @@ def sort_data_models(
         unsorted_data_model_names = set(unresolved_reference_model_names)
         for model in unresolved_references:
             unresolved_model = (
-                model.reference_classes - {model.name} - set(sorted_data_models)
+                model.reference_classes - {model.path} - set(sorted_data_models)
             )
             if not unresolved_model:
-                sorted_data_models[model.name] = model
+                sorted_data_models[model.path] = model
                 continue
             if not unresolved_model - unsorted_data_model_names:
-                sorted_data_models[model.name] = model
-                require_update_action_models.append(model.name)
+                sorted_data_models[model.path] = model
+                require_update_action_models.append(model.path)
                 continue
             # unresolved
             unresolved_classes = ', '.join(
@@ -192,6 +190,13 @@ def relative(current_module: str, reference: str) -> Tuple[str, str]:
     return left, right
 
 
+def get_most_of_parent(value: Any) -> Optional[Any]:
+    if not hasattr(value, 'parent'):
+        return value
+    parent = getattr(value, 'parent')
+    return get_most_of_parent(parent)
+
+
 class Result(BaseModel):
     body: str
     source: Optional[Path]
@@ -221,7 +226,7 @@ class Parser(ABC):
         custom_template_dir: Optional[Path] = None,
         extra_template_data: Optional[DefaultDict[str, Dict[str, Any]]] = None,
         target_python_version: PythonVersion = PythonVersion.PY_37,
-        dump_resolve_reference_action: Optional[Callable[[List[str]], str]] = None,
+        dump_resolve_reference_action: Optional[Callable[[Iterable[str]], str]] = None,
         validation: bool = False,
         field_constraints: bool = False,
         snake_case_field: bool = False,
@@ -253,7 +258,7 @@ class Parser(ABC):
         self.target_python_version: PythonVersion = target_python_version
         self.results: List[DataModel] = []
         self.dump_resolve_reference_action: Optional[
-            Callable[[List[str]], str]
+            Callable[[Iterable[str]], str]
         ] = dump_resolve_reference_action
         self.validation: bool = validation
         self.field_constraints: bool = field_constraints
@@ -365,9 +370,42 @@ class Parser(ABC):
             sorted(sorted_data_models.values(), key=module_key, reverse=True),
             key=module_key,
         )
+
+        module_models: List[Tuple[Tuple[str, ...], List[DataModel]]] = []
+
         for module, models in (
             (k, [*v]) for k, v in grouped_models
         ):  # type: Tuple[str, ...], List[DataModel]
+
+            # Remove duplicated name model
+            unique_model_names: Set[str] = set()
+            for model in models:
+                if model.name in unique_model_names:
+                    # Remove duplicated name model
+                    models.remove(model)
+                    continue
+                unique_model_names.add(model.name)
+            module_models.append((module, models,))
+
+            scoped_model_resolver = ModelResolver(
+                exclude_names={i.alias or i.import_ for m in models for i in m.imports},
+                duplicate_name_suffix='Model',
+            )
+
+            for model in models:
+                class_name: str = model.class_name
+                generated_name: str = scoped_model_resolver.add(
+                    model.path, class_name, unique=True, class_name=True
+                ).name
+                if class_name != generated_name:
+                    if '.' in model.reference.name:
+                        model.reference.name = (
+                            f"{model.reference.name.rsplit('.', 1)[0]}.{generated_name}"
+                        )
+                    else:
+                        model.reference.name = generated_name
+
+        for module, models in module_models:
             module_path = '.'.join(module)
 
             init = False
@@ -385,93 +423,69 @@ class Parser(ABC):
 
             result: List[str] = []
             imports = Imports()
-            models_to_update: List[str] = []
             scoped_model_resolver = ModelResolver()
             import_map: Dict[str, Tuple[str, str]] = {}
-            model_names: Set[str] = {m.name for m in models}
-            processed_models: Set[str] = set()
-            model_cache: Dict[Tuple[str, ...], DataModel] = {}
-            duplicated_models: Dict[str, DataModel] = {}
+
             for model in models:
                 alias_map: Dict[str, Optional[str]] = {}
-                if model.name in require_update_action_models:
-                    ref_names = {
-                        d.reference.name for d in model.all_data_types if d.reference
-                    }
-                    if model.name in ref_names or (
-                        not ref_names - model_names and ref_names - processed_models
-                    ):
-                        models_to_update += [model.name]
-                        processed_models.add(model.name)
                 imports.append(model.imports)
-                model.reference_classes = {
-                    r for r in model.reference_classes if r not in model_names
-                }
                 for data_type in model.all_data_types:
                     # To change from/import
 
-                    if not data_type.reference:
+                    if not data_type.reference or data_type.reference.source in models:
                         # No need to import non-reference model.
+                        # Or, Referenced model is in the same file. we don't need to import the model
                         continue
 
-                    if data_type.reference.source in models:
-                        # Referenced model is in the same file. we don't need to import the model
-                        data_type.alias = data_type.name
-                        continue
                     elif (
                         isinstance(self.source, Path)
                         and self.source.is_file()
                         and self.source.name == data_type.reference.path.split('#/')[0]
                     ):
-                        full_name = data_type.type
+                        # The input file is no need to import
+                        full_name = data_type.reference.name
                     else:
-                        full_name = data_type.full_name
-                    from_, import_ = relative(module_path, full_name)  # type: ignore
-                    name = data_type.name
-                    full_path = f'{from_}/{import_}'
-                    if full_path in alias_map:
-                        alias = alias_map[full_path] or import_
-                    else:
-                        alias = scoped_model_resolver.add(
-                            full_path.split('/'), import_, unique=True
-                        ).name
-                        alias_map[full_path] = None if alias == import_ else alias
-                    new_name = (
+                        full_name = data_type.full_name or data_type.reference.name
+                    from_, import_ = full_path = relative(module_path, full_name)
+
+                    alias = scoped_model_resolver.add(
+                        full_path, import_, unique=True
+                    ).name
+                    if alias != import_:
+                        alias_map[data_type.reference.path] = alias
+
+                    name = data_type.reference.short_name
+                    data_type.alias = (
                         f'{alias}.{name}'
                         if (from_ and import_ and alias != name)
                         else name
                     )
-                    if name in model.reference_classes:
-                        model.reference_classes.remove(name)
-                        model.reference_classes.add(new_name)
-                    data_type.alias = new_name
 
                     if (
                         full_name
                         and full_name.startswith(from_)
                         or f'.{full_name}'.startswith(from_)
                     ):
-                        import_map[new_name] = (
-                            f'.{full_name[:len(new_name) * - 1 - 1]}',  # type: ignore
-                            new_name.split('.')[0],
-                        )
-
-                for ref_name in model.reference_classes:
-                    if ref_name in import_map:
-                        from_, import_ = import_map[ref_name]
+                        import_map[data_type.reference.path] = full_path
+                for ref_path in model.reference_classes:
+                    reference: Reference = self.model_resolver.get(ref_path)  # type: ignore
+                    if reference.source in models:
+                        continue
+                    elif ref_path in import_map:
+                        from_, import_ = import_map[ref_path]
                     else:
-                        from_, import_ = relative(module_path, ref_name)
+                        from_, import_ = relative(module_path, reference.name)
                     if init:
                         from_ += "."
                     imports.append(
                         Import(
-                            from_=from_,
-                            import_=import_,
-                            alias=alias_map.get(f'{from_}/{import_}'),
+                            from_=from_, import_=import_, alias=alias_map.get(ref_path),
                         )
                     )
 
-                if self.reuse_model:
+            if self.reuse_model:
+                model_cache: Dict[Tuple[str, ...], Reference] = {}
+                for model in models:
                     model_key = tuple(
                         to_hashable(v)
                         for v in (
@@ -480,58 +494,76 @@ class Parser(ABC):
                             model.fields,
                         )
                     )
-                    cached_model = model_cache.get(model_key)
-                    if cached_model:
+                    cached_model_reference = model_cache.get(model_key)
+                    if cached_model_reference:
                         if isinstance(model, Enum):
-                            duplicated_models[model.name] = cached_model
+                            for child in model.reference.children:
+                                if isinstance(child, DataType):  # pragma: no cover
+                                    # child is resolved data_type by reference
+                                    data_model = get_most_of_parent(child)
+
+                                    # TODO: replace reference in all modules
+                                    if data_model in models:  # pragma: no cover
+                                        child.replace_reference(cached_model_reference)
                         else:
                             index = models.index(model)
                             inherited_model = model.__class__(
-                                name=model.name,
                                 fields=[],
-                                base_classes=[cached_model.name],
+                                base_classes=[cached_model_reference],
                                 description=model.description,
+                                reference=Reference(
+                                    name=model.name,
+                                    path=model.reference.path + '/reuse',
+                                ),
                             )
+                            if (
+                                cached_model_reference.path
+                                in require_update_action_models
+                            ):
+                                require_update_action_models.append(
+                                    inherited_model.path
+                                )
                             models.insert(index, inherited_model)
                             models.remove(model)
 
                     else:
-                        model_cache[model_key] = model
+                        model_cache[model_key] = model.reference
 
-            if self.reuse_model:
-                for model in models:
-                    for data_type in model.all_data_types:  # pragma: no cover
-                        if data_type.type:
-                            duplicated_model = duplicated_models.get(data_type.type)
-                            if duplicated_model:
-                                data_type.alias = duplicated_model.name
-                                data_type.reference = duplicated_model.reference
             if self.set_default_enum_member:
-                for model in models:  # pragma: no cover
+                for model in models:
                     for model_field in model.fields:
-                        if model_field.default:
-                            for data_type in model_field.data_type.all_data_types:
-                                if data_type.reference:
-                                    if isinstance(data_type.reference.source, Enum):
-                                        enum_member = data_type.reference.source.find_member(
-                                            model_field.default
-                                        )
-                                        if enum_member:
-                                            model_field.default = enum_member
+                        if not model_field.default:
+                            continue
+                        for data_type in model_field.data_type.all_data_types:
+                            if data_type.reference and isinstance(
+                                data_type.reference.source, Enum
+                            ):  # pragma: no cover
+                                enum_member = data_type.reference.source.find_member(
+                                    model_field.default
+                                )
+                                if enum_member:
+                                    model_field.default = enum_member
             if with_import:
-                result += [str(imports), str(self.imports), '\n']
+                result += [str(self.imports), str(imports), '\n']
 
             code = dump_templates(models)
             result += [code]
 
             if self.dump_resolve_reference_action is not None:
-                result += ['\n', self.dump_resolve_reference_action(models_to_update)]
+                result += [
+                    '\n',
+                    self.dump_resolve_reference_action(
+                        m.reference.short_name
+                        for m in models
+                        if m.path in require_update_action_models
+                    ),
+                ]
 
             body = '\n'.join(result)
             if code_formatter:
                 body = code_formatter.format_code(body)
 
-            results[module] = Result(body=body, source=models[0].path)
+            results[module] = Result(body=body, source=models[0].file_path)
 
         # retain existing behaviour
         if [*results] == [('__init__.py',)]:
