@@ -9,6 +9,7 @@ from typing import (
     DefaultDict,
     Dict,
     Generator,
+    Iterable,
     List,
     Mapping,
     Optional,
@@ -34,7 +35,7 @@ from datamodel_code_generator.parser import LiteralType
 
 from ..model import pydantic as pydantic_model
 from ..parser.base import Parser
-from ..reference import is_url
+from ..reference import Reference, is_url
 from ..types import DataType, DataTypeManager, Types
 
 
@@ -131,7 +132,9 @@ class JsonSchemaObject(BaseModel):
     @validator('ref')
     def validate_ref(cls, value: Any) -> Any:
         if isinstance(value, str) and '#' in value:
-            if '#/' in value or value[0] == '#' or value[-1] == '#':
+            if value.endswith('#/'):
+                return value[:-1]
+            elif '#/' in value or value[0] == '#' or value[-1] == '#':
                 return value
             return value.replace('#', '#/')
         return value
@@ -232,7 +235,7 @@ class JsonSchemaParser(Parser):
         custom_template_dir: Optional[Path] = None,
         extra_template_data: Optional[DefaultDict[str, Dict[str, Any]]] = None,
         target_python_version: PythonVersion = PythonVersion.PY_37,
-        dump_resolve_reference_action: Optional[Callable[[List[str]], str]] = None,
+        dump_resolve_reference_action: Optional[Callable[[Iterable[str]], str]] = None,
         validation: bool = False,
         field_constraints: bool = False,
         snake_case_field: bool = False,
@@ -343,7 +346,7 @@ class JsonSchemaParser(Parser):
 
     def get_ref_data_type(self, ref: str) -> DataType:
         reference = self.model_resolver.add_ref(ref)
-        return self.data_type.from_reference(reference)
+        return self.data_type(reference=reference)
 
     def set_additional_properties(self, name: str, obj: JsonSchemaObject) -> None:
         if obj.additionalProperties:
@@ -359,33 +362,33 @@ class JsonSchemaParser(Parser):
     def parse_list_item(
         self, name: str, target_items: List[JsonSchemaObject], path: List[str]
     ) -> DataType:
-        data_types: List[DataType] = []
-        for index, item in enumerate(target_items):
+        def parse_item(index: int, item: JsonSchemaObject) -> DataType:
             if item.ref:  # $ref
-                data_types.append(self.get_ref_data_type(item.ref))
+                return self.get_ref_data_type(item.ref)
             elif not any(v for k, v in vars(item).items() if k != 'type'):
                 # trivial types
-                data_types.append(self.get_data_type(item))
+                return self.get_data_type(item)
             elif (
                 item.is_array
                 and isinstance(item.items, JsonSchemaObject)
                 and not any(v for k, v in vars(item.items).items() if k != 'type')
             ):
                 # trivial item types
-                data_types.append(
-                    self.data_type(
-                        data_types=[self.get_data_type(item.items)], is_list=True,
-                    )
+                return self.data_type(
+                    data_types=[self.get_data_type(item.items)], is_list=True,
                 )
             elif item.is_object:
-                data_types.append(
-                    self.parse_object(
-                        name, item, [*path, str(index)], singular_name=True,
-                    )
+                return self.parse_object(
+                    name, item, [*path, str(index)], singular_name=True,
                 )
             else:
-                data_types.append(self.get_data_type(item))
-        return self.data_type(data_types=data_types)
+                return self.get_data_type(item)
+
+        return self.data_type(
+            data_types=[
+                parse_item(index, item) for index, item in enumerate(target_items)
+            ]
+        )
 
     def parse_any_of(
         self, name: str, obj: JsonSchemaObject, path: List[str]
@@ -405,7 +408,7 @@ class JsonSchemaParser(Parser):
         ignore_duplicate_model: bool = False,
     ) -> DataType:
         fields: List[DataModelFieldBase] = []
-        base_classes: List[DataType] = []
+        base_classes: List[Reference] = []
         if len(obj.allOf) == 1:
             single_obj = obj.allOf[0]
             if single_obj.ref and single_obj.ref_type == JSONReference.LOCAL:
@@ -415,35 +418,29 @@ class JsonSchemaParser(Parser):
                     return self.get_ref_data_type(single_obj.ref)
         for all_of_item in obj.allOf:
             if all_of_item.ref:  # $ref
-                base_classes.append(self.get_ref_data_type(all_of_item.ref))
+                base_classes.append(self.model_resolver.add_ref(all_of_item.ref))
             else:
                 fields.extend(self.parse_object_fields(all_of_item, path))
         # ignore an undetected object
         if ignore_duplicate_model and not fields and len(base_classes) == 1:
-            return base_classes[0]
+            return self.data_type(reference=base_classes[0])
         reference = self.model_resolver.add(
             path, name, class_name=True, unique=True, loaded=True
         )
-        class_name = reference.name
-        self.set_additional_properties(class_name, obj)
+        self.set_additional_properties(reference.name, obj)
         data_model_type = self.data_model_type(
-            class_name,
+            reference=reference,
             fields=fields,
-            base_classes=[b.type_hint for b in base_classes],
-            auto_import=False,
+            base_classes=base_classes,
             custom_base_class=self.base_class,
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
-            reference=reference,
         )
-        # add imports for the fields
-        for f in fields:
-            data_model_type.imports.extend(f.imports)
         self.append_result(data_model_type)
 
-        return self.data_type.from_reference(reference)
+        return self.data_type(reference=reference)
 
     def parse_object_fields(
         self, obj: JsonSchemaObject, path: List[str]
@@ -490,8 +487,8 @@ class JsonSchemaParser(Parser):
                         isinstance(field.additionalProperties.items, JsonSchemaObject)
                         and field.additionalProperties.items.ref
                     ):
-                        additional_properties_type = self.data_type.from_reference(
-                            self.model_resolver.add_ref(
+                        additional_properties_type = self.data_type(
+                            reference=self.model_resolver.add_ref(
                                 field.additionalProperties.items.ref,
                             ),
                             is_list=True,
@@ -515,8 +512,8 @@ class JsonSchemaParser(Parser):
                         if self.should_parse_enum_as_literal(
                             field.additionalProperties
                         ):
-                            additional_properties_type = self.data_type.create_literal(
-                                field.additionalProperties.enum
+                            additional_properties_type = self.data_type(
+                                literals=field.additionalProperties.enum
                             )
                         else:
                             additional_properties_type = self.parse_enum(
@@ -562,9 +559,7 @@ class JsonSchemaParser(Parser):
                     field_type = self.data_type_manager.get_data_type(Types.object)
             elif field.enum:
                 if self.should_parse_enum_as_literal(field):
-                    field_type = self.data_type_manager.data_type.create_literal(
-                        field.enum
-                    )
+                    field_type = self.data_type(literals=field.enum)
                 else:
                     field_type = self.parse_enum(field_name, field, [*path, field_name])
             else:
@@ -623,82 +618,63 @@ class JsonSchemaParser(Parser):
         self.set_title(class_name, obj)
         self.set_additional_properties(class_name, additional_properties or obj)
         data_model_type = self.data_model_type(
-            class_name,
+            reference=reference,
             fields=self.parse_object_fields(obj, path),
             custom_base_class=self.base_class,
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
-            reference=reference,
         )
         self.append_result(data_model_type)
-        return self.data_type.from_reference(reference)
+        return self.data_type(reference=reference)
 
     def parse_array_fields(
         self, name: str, obj: JsonSchemaObject, path: List[str]
     ) -> DataModelFieldBase:
-        if isinstance(obj.items, JsonSchemaObject):
-            items = [obj.items]
-        else:
-            items = obj.items or []
-        item_obj_data_types: List[DataType] = []
-        for index, item in enumerate(items):
+        def parse_field(index: int, item: JsonSchemaObject) -> DataType:
             field_path = [*path, str(index)]
             if item.has_constraint and (obj.has_constraint or self.field_constraints):
-                item_obj_data_types.append(
-                    self.parse_root_type(
-                        self.model_resolver.add(
-                            field_path,
-                            name,
-                            class_name=True,
-                            singular_name=True,
-                            unique=True,
-                        ).name,
-                        item,
+                return self.parse_root_type(
+                    self.model_resolver.add(
                         field_path,
-                    )
+                        name,
+                        class_name=True,
+                        singular_name=True,
+                        unique=True,
+                    ).name,
+                    item,
+                    field_path,
                 )
             elif item.ref:
-                item_obj_data_types.append(self.get_ref_data_type(item.ref))
+                return self.get_ref_data_type(item.ref)
             elif isinstance(item, JsonSchemaObject) and item.properties:
-                item_obj_data_types.append(
-                    self.parse_object(name, item, field_path, singular_name=True,)
-                )
+                return self.parse_object(name, item, field_path, singular_name=True,)
             elif item.anyOf:
-                item_obj_data_types.append(self.parse_any_of(name, item, field_path))
+                return self.parse_any_of(name, item, field_path)
             elif item.oneOf:
-                item_obj_data_types.append(self.parse_one_of(name, item, field_path))
+                return self.parse_one_of(name, item, field_path)
             elif item.allOf:
-                item_obj_data_types.append(
-                    self.parse_all_of(
-                        self.model_resolver.add(
-                            field_path, name, singular_name=True
-                        ).name,
-                        item,
-                        field_path,
-                        ignore_duplicate_model=True,
-                    )
+                return self.parse_all_of(
+                    self.model_resolver.add(field_path, name, singular_name=True).name,
+                    item,
+                    field_path,
+                    ignore_duplicate_model=True,
                 )
             elif item.enum:
                 if self.should_parse_enum_as_literal(item):
-                    item_obj_data_types.append(
-                        self.data_type_manager.data_type.create_literal(item.enum)
-                    )
+                    return self.data_type(literals=item.enum)
                 else:
-                    item_obj_data_types.append(
-                        self.parse_enum(name, item, field_path, singular_name=True)
-                    )
+                    return self.parse_enum(name, item, field_path, singular_name=True)
             elif item.is_array:
-                item_obj_data_types.append(
-                    self.parse_array_fields(
-                        self.model_resolver.add(field_path, name, class_name=True).name,
-                        item,
-                        field_path,
-                    ).data_type
-                )
+                return self.parse_array_fields(
+                    self.model_resolver.add(field_path, name, class_name=True).name,
+                    item,
+                    field_path,
+                ).data_type
             else:
-                item_obj_data_types.append(self.get_data_type(item))
+                return self.get_data_type(item)
+
         if self.force_optional_for_required_fields:
             required: bool = False
             nullable: Optional[bool] = None
@@ -712,7 +688,17 @@ class JsonSchemaParser(Parser):
                 required = not obj.nullable and required
                 nullable = None
         return self.data_model_field_type(
-            data_type=self.data_type(data_types=item_obj_data_types, is_list=True,),
+            data_type=self.data_type(
+                data_types=[
+                    parse_field(index, item)
+                    for index, item in enumerate(
+                        [obj.items]
+                        if isinstance(obj.items, JsonSchemaObject)
+                        else obj.items or []
+                    )
+                ],
+                is_list=True,
+            ),
             example=obj.example,
             examples=obj.examples,
             default=obj.default,
@@ -729,17 +715,16 @@ class JsonSchemaParser(Parser):
         field = self.parse_array_fields(name, obj, [*path, name])
         reference = self.model_resolver.add(path, name, loaded=True)
         data_model_root = self.data_model_root_type(
-            name,
-            [field],
+            reference=reference,
+            fields=[field],
             custom_base_class=self.base_class,
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
-            reference=reference,
         )
         self.append_result(data_model_root)
-        return self.data_type.from_reference(reference)
+        return self.data_type(reference=reference)
 
     def parse_root_type(
         self,
@@ -768,8 +753,8 @@ class JsonSchemaParser(Parser):
         self.set_title(name, obj)
         self.set_additional_properties(name, additional_properties or obj)
         data_model_root_type = self.data_model_root_type(
-            name,
-            [
+            reference=reference,
+            fields=[
                 self.data_model_field_type(
                     data_type=data_type,
                     description=obj.description,
@@ -785,10 +770,9 @@ class JsonSchemaParser(Parser):
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
-            reference=reference,
         )
         self.append_result(data_model_root_type)
-        return self.data_type.from_reference(reference)
+        return self.data_type(reference=reference)
 
     def parse_enum(
         self,
@@ -849,14 +833,13 @@ class JsonSchemaParser(Parser):
                 loaded=True,
             )
             enum = Enum(
-                reference.name,
+                reference=reference,
                 fields=enum_fields,
                 path=self.current_source_path,
                 description=obj.description if self.use_schema_description else None,
-                reference=reference,
             )
             self.append_result(enum)
-            return self.data_type.from_reference(reference)
+            return self.data_type(reference=reference)
 
         root_reference = self.model_resolver.add(
             path,
@@ -877,18 +860,17 @@ class JsonSchemaParser(Parser):
             loaded=True,
         )
         enum = Enum(
-            enum_reference.name,
+            reference=enum_reference,
             fields=enum_fields,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
-            reference=enum_reference,
         )
         self.append_result(enum)
         data_model_root_type = self.data_model_root_type(
-            name,
-            [
+            reference=root_reference,
+            fields=[
                 self.data_model_field_type(
-                    data_type=self.data_type.from_reference(enum_reference),
+                    data_type=self.data_type(reference=enum_reference),
                     description=obj.description,
                     example=obj.example,
                     examples=obj.examples,
@@ -901,17 +883,18 @@ class JsonSchemaParser(Parser):
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
-            reference=root_reference,
         )
         self.append_result(data_model_root_type)
-        return self.data_type.from_reference(root_reference)
+        return self.data_type(reference=root_reference)
 
-    def _get_ref_body(self, ref: str) -> Dict[Any, Any]:
-        if is_url(ref):
-            return self._get_ref_body_from_url(ref)
-        return self._get_ref_body_from_remote(ref)
+    def _get_ref_body(self, resolved_ref: str) -> Dict[Any, Any]:
+        if is_url(resolved_ref):
+            return self._get_ref_body_from_url(resolved_ref)
+        return self._get_ref_body_from_remote(resolved_ref)
 
     def _get_ref_body_from_url(self, ref: str) -> Dict[Any, Any]:
+        if ref[-1] == '#':
+            ref = ref[:-1]
         cached_ref_body: Optional[Dict[str, Any]] = self.remote_object_cache.get(ref)
         if cached_ref_body:
             return cached_ref_body
@@ -929,14 +912,12 @@ class JsonSchemaParser(Parser):
         self.remote_object_cache[ref] = ref_body
         return ref_body
 
-    def _get_ref_body_from_remote(self, ref: str) -> Dict[Any, Any]:
+    def _get_ref_body_from_remote(self, resolved_ref: str) -> Dict[Any, Any]:
         # Remote Reference – $ref: 'document.json' Uses the whole document located on the same server and in
         # the same location. TODO treat edge case
-        if self.current_source_path and len(self.current_source_path.parts) > 1:
-            full_path = self.base_path / self.current_source_path.parent / ref
-        else:
-            full_path = self.base_path / ref
-
+        if resolved_ref[-1] == '#':
+            resolved_ref = resolved_ref[:-1]
+        full_path = self.base_path / resolved_ref
         ref_full_path = str(full_path)
 
         cached_ref_body: Optional[Dict[str, Any]] = self.remote_object_cache.get(
@@ -956,42 +937,37 @@ class JsonSchemaParser(Parser):
             reference = self.model_resolver.add_ref(obj.ref)
             if not reference or not reference.loaded:
                 # https://swagger.io/docs/specification/using-ref/
+                ref = self.model_resolver.resolve_ref(obj.ref)
                 if obj.ref_type == JSONReference.LOCAL:
                     # Local Reference – $ref: '#/definitions/myElement'
-                    self.reserved_refs[tuple(self.model_resolver.current_root)].add(obj.ref)  # type: ignore
+                    self.reserved_refs[tuple(self.model_resolver.current_root)].add(ref)  # type: ignore
                 elif self.model_resolver.is_after_load(obj.ref):
-                    self.reserved_refs[tuple(obj.ref.split('#')[0].split('/'))].add(obj.ref)  # type: ignore
+                    self.reserved_refs[tuple(ref.split('#')[0].split('/'))].add(ref)  # type: ignore
                 else:
-                    if obj.ref_type == JSONReference.REMOTE and self.root_id_base_path:
-                        ref = f'{self.root_id_base_path}/{obj.ref}'
-                    else:
-                        ref = obj.ref
-                    if self.model_resolver.is_external_ref(ref):
-                        relative_path, object_path = ref.split('#/')
-                    elif self.model_resolver.is_external_root_ref(ref):
+                    if self.model_resolver.is_external_root_ref(ref):
                         relative_path, object_path = ref[:-1], ''
                     else:
-                        relative_path, object_path = ref, ''
+                        relative_path, object_path = ref.split('#')
 
                     models = self._get_ref_body(relative_path)
                     model_name = self.model_resolver.add_ref(obj.ref).name
-                    if obj.ref_type == JSONReference.REMOTE:
-                        previous_base_path: Optional[Path] = self.base_path
-                        self.base_path = (self.base_path / relative_path).parent
+
+                    if is_url(ref):
+                        relative_path, object_path = ref.split('#')
+                        relative_paths = [relative_path]
                     else:
-                        previous_base_path = None
-                    relative_paths = relative_path.split('/')
+                        if self.model_resolver.is_external_root_ref(ref):
+                            relative_path, object_path = ref[:-1], ''
+                        else:
+                            relative_path, object_path = ref.split('#')
+                        relative_paths = relative_path.split('/')
                     self._parse_file(
-                        models,
+                        self._get_ref_body(relative_path),
                         model_name,
                         relative_paths,
                         object_path.split('/') if object_path else None,
                     )
-                    if previous_base_path:
-                        self.base_path = previous_base_path
-                    self.model_resolver.add_ref(
-                        ref, actual_module_name=''
-                    ).loaded = True
+                    self.model_resolver.add_ref(obj.ref,).loaded = True
 
         if obj.items:
             if isinstance(obj.items, JsonSchemaObject):
@@ -1036,7 +1012,8 @@ class JsonSchemaParser(Parser):
         previous_root_id: Optional[str] = self.root_id
         if root_id:
             try:
-                self._get_ref_body(root_id)
+                resolved_ref = self.model_resolver.resolve_ref(root_id)
+                self._get_ref_body(resolved_ref)
             except Exception as e:
                 print(f'Parse $id failed. $id={root_id}\n {str(e)}', file=sys.stderr)
                 self.root_id = None
@@ -1102,14 +1079,7 @@ class JsonSchemaParser(Parser):
 
             with self.model_resolver.current_root_context(path_parts):
                 for reserved_ref in sorted(reserved_refs):
-                    if self.model_resolver.add_ref(reserved_ref).loaded:
-                        continue
-                    file_path, model_path = reserved_ref.split('#', 1)
-                    valid_path_parts = [p for p in model_path.split('/') if p]
-                    if (
-                        len(valid_path_parts) == 1
-                        and self.model_resolver.add_ref(f'{file_path}#').loaded
-                    ):
+                    if self.model_resolver.add_ref(reserved_ref, resolved=True).loaded:
                         continue
                     # for root model
                     self.raw_obj = load_yaml(source.text)
@@ -1140,13 +1110,11 @@ class JsonSchemaParser(Parser):
         path_parts: List[str],
         object_paths: Optional[List[str]] = None,
     ) -> None:
+        object_paths = [o for o in object_paths or [] if o]
         if object_paths:
             path = [*path_parts, f'#/{object_paths[0]}', *object_paths[1:]]
         else:
             path = path_parts
-        reference = self.model_resolver.get(path)
-        if reference and reference.loaded:
-            return
         with self.model_resolver.current_root_context(path_parts):
             obj_name = self.model_resolver.add(path, obj_name, unique=False).name
             with self.root_id_context(raw):
