@@ -5,6 +5,9 @@ from functools import lru_cache
 from keyword import iskeyword
 from pathlib import Path
 from typing import (
+    TYPE_CHECKING,
+    Any,
+    ClassVar,
     DefaultDict,
     Dict,
     Generator,
@@ -12,29 +15,76 @@ from typing import (
     Mapping,
     Optional,
     Pattern,
+    Sequence,
     Set,
     Tuple,
     Union,
 )
 
 import inflect
-from pydantic import BaseModel
+from pydantic import BaseModel, validator
 
 from datamodel_code_generator import cached_property
 
+if TYPE_CHECKING:
+    from pydantic.typing import AbstractSetIntStr, DictStrAny, MappingIntStrAny
 
-class Reference(BaseModel):
+
+class _BaseModel(BaseModel):
+    _exclude_fields: ClassVar[Set[str]] = set()
+    _pass_fields: ClassVar[Set[str]] = set()
+
+    def __init__(self, **values: Any) -> None:  # type: ignore
+        super().__init__(**values)
+        for pass_field_name in self._pass_fields:
+            if pass_field_name in values:
+                setattr(self, pass_field_name, values[pass_field_name])
+
+    def dict(
+        self,
+        *,
+        include: Union['AbstractSetIntStr', 'MappingIntStrAny'] = None,
+        exclude: Union['AbstractSetIntStr', 'MappingIntStrAny'] = None,
+        by_alias: bool = False,
+        skip_defaults: bool = None,
+        exclude_unset: bool = False,
+        exclude_defaults: bool = False,
+        exclude_none: bool = False,
+    ) -> 'DictStrAny':
+        return super().dict(
+            include=include,
+            exclude=set(exclude or ()) | self._exclude_fields,
+            by_alias=by_alias,
+            skip_defaults=skip_defaults,
+            exclude_unset=exclude_unset,
+            exclude_defaults=exclude_defaults,
+            exclude_none=exclude_none,
+        )
+
+
+class Reference(_BaseModel):
     path: str
-    original_name: str
+    original_name: str = ''
     name: str
     loaded: bool = True
-    actual_module_name: Optional[str]
+    source: Optional[Any] = None
+    children: List[Any] = []
+    _exclude_fields: ClassVar = {'children'}
+
+    @validator('original_name')
+    def validate_original_name(cls, v: Any, values: Dict[str, Any]) -> str:
+        """
+        If original_name is empty then, `original_name` is assigned `name`
+        """
+        if v:  # pragma: no cover
+            return v
+        return values.get('name', v)  # pragma: no cover
 
     class Config:
         arbitrary_types_allowed = True
         keep_untouched = (cached_property,)
 
-    @cached_property
+    @property
     def module_name(self) -> Optional[str]:
         if is_url(self.path):  # pragma: no cover
             return None
@@ -45,35 +95,48 @@ class Reference(BaseModel):
         module_name = f'{".".join(path.parts[:-1])}.{path.stem.split(".")[0]}'
         if module_name.startswith(f'.{self.name.split(".", 1)[0]}'):
             return None
-        elif module_name == '.':
+        elif module_name == '.':  # pragma: no cover
             return None
         return module_name
 
+    @property
+    def short_name(self) -> str:
+        return self.name.rsplit('.', 1)[-1]
+
 
 ID_PATTERN: Pattern[str] = re.compile(r'^#[^/].*')
 
 
 class ModelResolver:
-    def __init__(self, aliases: Optional[Mapping[str, str]] = None) -> None:
+    def __init__(
+        self,
+        aliases: Optional[Mapping[str, str]] = None,
+        exclude_names: Set[str] = None,
+        duplicate_name_suffix: Optional[str] = None,
+    ) -> None:
         self.references: Dict[str, Reference] = {}
         self.aliases: Mapping[str, str] = {} if aliases is None else {**aliases}
-        self._current_root: List[str] = []
+        self._current_root: Sequence[str] = []
         self._root_id_base_path: Optional[str] = None
         self.ids: DefaultDict[str, Dict[str, str]] = defaultdict(dict)
         self.after_load_files: Set[str] = set()
+        self.exclude_names: Set[str] = exclude_names or set()
+        self.duplicate_name_suffix: Optional[str] = duplicate_name_suffix
 
     @property
-    def current_root(self) -> List[str]:
+    def current_root(self) -> Sequence[str]:
+        if len(self._current_root) > 1:
+            return self._current_root
         return self._current_root
 
-    def set_current_root(self, current_root: List[str]) -> None:
+    def set_current_root(self, current_root: Sequence[str]) -> None:
         self._current_root = current_root
 
     @contextmanager
     def current_root_context(
-        self, current_root: List[str]
+        self, current_root: Sequence[str]
     ) -> Generator[None, None, None]:
-        previous_root_path: List[str] = self.current_root
+        previous_root_path: Sequence[str] = self.current_root
         self.set_current_root(current_root)
         yield
         self.set_current_root(previous_root_path)
@@ -85,72 +148,82 @@ class ModelResolver:
     def set_root_id_base_path(self, root_id_base_path: Optional[str]) -> None:
         self._root_id_base_path = root_id_base_path
 
-    def add_id(self, id_: str, path: List[str]) -> None:
-        self.ids['/'.join(self.current_root)][id_] = self._get_path(path)
+    def add_id(self, id_: str, path: Sequence[str]) -> None:
+        self.ids['/'.join(self.current_root)][id_] = self.resolve_ref(path)
 
-    def _get_path(self, path: List[str]) -> str:
-        joined_path = '/'.join(p for p in path if p).replace('/#', '#')
+    def resolve_ref(self, path: Union[Sequence[str], str]) -> str:
+        if isinstance(path, str):
+            joined_path = path
+        else:
+            joined_path = self.join_path(path)
         if ID_PATTERN.match(joined_path):
             return self.ids['/'.join(self.current_root)][joined_path]
         elif '#' in joined_path:
+            if joined_path[0] == '#':
+                joined_path = f'{"/".join(self.current_root)}{joined_path}'
+            if self.is_remote_ref(joined_path):
+                return f'{"/".join(self.current_root[:-1])}/{joined_path}'
             delimiter = joined_path.index('#')
             return f"{''.join(joined_path[:delimiter])}#{''.join(joined_path[delimiter + 1:])}"
         elif self.root_id_base_path and self.current_root != path:
             return f'{self.root_id_base_path}/{joined_path}#'
-        return f'{joined_path}#'
+        joined_path = f'{joined_path}#'
+        if self.is_remote_ref(joined_path):
+            return f'{"/".join(self.current_root[:-1])}/{joined_path}'
+        return joined_path
+
+    def is_remote_ref(self, resolved_ref: str) -> bool:
+        return (
+            self.is_external_ref(resolved_ref)
+            and not is_url(resolved_ref)
+            and len(self.current_root) > 1
+        )
 
     def is_after_load(self, ref: str) -> bool:
-        if self.current_root and len(self.current_root) > 1:
-            ref = f"{'/'.join(self.current_root[:-1])}/{ref}"
-        if self.is_external_ref(ref):
-            return ref.split('#/', 1)[0] in self.after_load_files
-        elif self.is_external_root_ref(ref):
+        ref = self.resolve_ref(ref)
+        if self.is_external_root_ref(ref):
             return ref[:-1] in self.after_load_files
-        return False
+        elif self.is_external_ref(ref):
+            return ref.split('#/', 1)[0] in self.after_load_files
+        return False  # pragma: no cover
 
     @staticmethod
     def is_external_ref(ref: str) -> bool:
-        return '#/' in ref
+        return '#' in ref and ref[0] != '#'
 
     @staticmethod
     def is_external_root_ref(ref: str) -> bool:
         return ref[-1] == '#'
 
-    def add_ref(self, ref: str, actual_module_name: Optional[str] = None) -> Reference:
-        path = self._get_path(ref.split('/'))
+    @staticmethod
+    def join_path(path: Sequence[str]) -> str:
+        joined_path = '/'.join(p for p in path if p).replace('/#', '#')
+        if '#' not in joined_path:
+            joined_path += '#'
+        return joined_path
+
+    def add_ref(self, ref: str, resolved: bool = False) -> Reference:
+        if not resolved:
+            path = self.resolve_ref(ref)
+        else:
+            path = ref
         reference = self.references.get(path)
         if reference:
-            reference.actual_module_name = actual_module_name
             return reference
         split_ref = ref.rsplit('/', 1)
         if len(split_ref) == 1:
-            parents = self.root_id_base_path
             original_name = Path(
                 split_ref[0][:-1] if self.is_external_root_ref(ref) else split_ref[0]
             ).stem
         else:
-            parents = split_ref[0]
             original_name = (
                 Path(split_ref[1][:-1]).stem
                 if self.is_external_root_ref(ref)
                 else split_ref[1]
             )
-        if self.is_after_load(ref):
-            loaded: bool = False
-        else:
-            loaded = not self.is_external_ref(ref) and not self.is_external_root_ref(
-                ref
-            )
-        if not original_name:
-            original_name = Path(parents).stem  # type: ignore
-            loaded = False
         name = self.get_class_name(original_name, unique=False)
         reference = Reference(
-            path=path,
-            original_name=original_name,
-            name=name,
-            loaded=loaded,
-            actual_module_name=actual_module_name,
+            path=path, original_name=original_name, name=name, loaded=False,
         )
 
         self.references[path] = reference
@@ -158,7 +231,7 @@ class ModelResolver:
 
     def add(
         self,
-        path: List[str],
+        path: Sequence[str],
         original_name: str,
         *,
         class_name: bool = False,
@@ -167,14 +240,17 @@ class ModelResolver:
         singular_name_suffix: str = 'Item',
         loaded: bool = False,
     ) -> Reference:
-        joined_path: str = self._get_path(path)
-        if joined_path in self.references:
-            reference = self.references[joined_path]
+        joined_path = self.join_path(path)
+        reference: Optional[Reference] = self.references.get(joined_path)
+        if reference:
             if loaded and not reference.loaded:
                 reference.loaded = True
-            return reference
-        elif not original_name:
-            original_name = Path(joined_path.split('#')[0]).stem
+            if (
+                not original_name
+                or original_name == reference.original_name
+                or original_name == reference.name
+            ):
+                return reference
         name = original_name
         if singular_name:
             name = get_singular_name(name, singular_name_suffix)
@@ -182,18 +258,19 @@ class ModelResolver:
             name = self.get_class_name(name, unique)
         elif unique:
             name = self._get_uniq_name(name)
-        reference = Reference(
-            path=joined_path, original_name=original_name, name=name, loaded=loaded
-        )
-        self.references[joined_path] = reference
+        if reference:
+            reference.original_name = original_name
+            reference.name = name
+            reference.loaded = loaded
+        else:
+            reference = Reference(
+                path=joined_path, original_name=original_name, name=name, loaded=loaded
+            )
+            self.references[joined_path] = reference
         return reference
 
-    def get(
-        self, path: Union[List[str], str]
-    ) -> Optional[Reference]:  # pragma: no cover
-        if isinstance(path, str):
-            return self.references.get(path)
-        return self.references[self._get_path(path)]
+    def get(self, path: Union[Sequence[str], str]) -> Optional[Reference]:
+        return self.references.get(self.resolve_ref(path))
 
     def get_class_name(self, field_name: str, unique: bool = True) -> str:
         if '.' in field_name:
@@ -215,12 +292,20 @@ class ModelResolver:
     def _get_uniq_name(self, name: str, camel: bool = False) -> str:
         uniq_name: str = name
         count: int = 1
-        reference_names = {r.name for r in self.references.values()}
+        reference_names = {
+            r.name for r in self.references.values()
+        } | self.exclude_names
         while uniq_name in reference_names:
-            if camel:
-                uniq_name = f'{name}{count}'
+            if self.duplicate_name_suffix:
+                name_parts: List[Union[str, int]] = [
+                    name,
+                    self.duplicate_name_suffix,
+                    count - 1,
+                ]
             else:
-                uniq_name = f'{name}_{count}'
+                name_parts = [name, count]
+            delimiter = '' if camel else '_'
+            uniq_name = delimiter.join(str(p) for p in name_parts if p)
             count += 1
         return uniq_name
 
