@@ -1,8 +1,20 @@
 from abc import ABC, abstractmethod
 from enum import Enum, auto
-from typing import Any, Dict, Iterator, List, Optional, Set, Tuple, Type
-
-from pydantic import BaseModel
+from itertools import chain
+from typing import (
+    Any,
+    ClassVar,
+    Dict,
+    FrozenSet,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+)
 
 from datamodel_code_generator.format import PythonVersion
 from datamodel_code_generator.imports import (
@@ -17,19 +29,23 @@ from datamodel_code_generator.imports import (
     IMPORT_UNION,
     Import,
 )
-from datamodel_code_generator.reference import Reference
+from datamodel_code_generator.reference import Reference, _BaseModel
+
+T = TypeVar('T')
 
 
-class DataType(BaseModel):
+def chain_as_tuple(*iterables: Iterable[T]) -> Tuple[T, ...]:
+    return tuple(chain(*iterables))
+
+
+class DataType(_BaseModel):
     type: Optional[str]
     reference: Optional[Reference]
     data_types: List['DataType'] = []
     is_func: bool = False
     kwargs: Optional[Dict[str, Any]]
-    imports_: List[Import] = []
+    imports: List[Import] = []
     python_version: PythonVersion = PythonVersion.PY_37
-    unresolved_types: Set[str] = {*()}
-    ref: bool = False
     is_optional: bool = False
     is_dict: bool = False
     is_list: bool = False
@@ -37,16 +53,34 @@ class DataType(BaseModel):
     use_standard_collections: bool = False
     use_generic_container: bool = False
     alias: Optional[str] = None
+    parent: Optional[Any] = None
+    children: List[Any] = []
 
-    @classmethod
-    def from_reference(cls, reference: Reference, is_list: bool = False) -> 'DataType':
-        data_type = cls(type=reference.name, ref=True, is_list=is_list)
-        data_type.reference = reference
-        return data_type
+    _exclude_fields: ClassVar[Set[str]] = {'parent', 'children'}
+    _pass_fields: ClassVar[Set[str]] = {'parent', 'children', 'data_types', 'reference'}
 
-    @classmethod
-    def create_literal(cls, literals: List[str]) -> 'DataType':
-        return cls(literals=literals)
+    @property
+    def unresolved_types(self) -> FrozenSet[str]:
+        return frozenset(
+            {
+                t.reference.path
+                for data_types in self.data_types
+                for t in data_types.all_data_types
+                if t.reference
+            }
+            | ({self.reference.path} if self.reference else set())
+        )
+
+    def replace_reference(self, reference: Reference) -> None:
+        if not self.reference:  # pragma: no cover
+            raise Exception(
+                f'`{self.__class__.__name__}.replace_reference()` can\'t be called'
+                f' when `reference` field is empty.'
+            )
+
+        self.reference.children.remove(self)
+        self.reference = reference
+        reference.children.append(self)
 
     @property
     def module_name(self) -> Optional[str]:
@@ -59,20 +93,21 @@ class DataType(BaseModel):
             return f'{module_name}.{self.type}'
         return self.type  # type: ignore
 
-    @property
-    def name(self) -> str:
-        return self.type.rsplit('.', 1)[-1]  # type: ignore
-
     @property
     def all_data_types(self) -> Iterator['DataType']:
         for data_type in self.data_types:
             yield from data_type.all_data_types
         yield self
 
-    def __init__(self, **values: Any) -> None:  # type: ignore
-        super().__init__(**values)
-        if self.type and (self.reference or self.ref):
-            self.unresolved_types.add(self.type)
+    @property
+    def all_imports(self) -> Iterator[Import]:
+        for data_type in self.data_types:
+            yield from data_type.all_imports
+        yield from self.imports
+
+    def __init__(self, **values: Any) -> None:
+        super().__init__(**values)  # type: ignore
+
         for type_ in self.data_types:
             if type_.type == 'Any' and type_.is_optional:
                 if any(
@@ -111,21 +146,20 @@ class DataType(BaseModel):
                 (self.is_dict, IMPORT_DICT),
             )
         for field, import_ in imports:
-            if field and import_ not in self.imports_:
-                self.imports_.append(import_)
+            if field and import_ not in self.imports:
+                self.imports.append(import_)
+
         for data_type in self.data_types:
-            self.imports_.extend(data_type.imports_)
-            self.unresolved_types.update(data_type.unresolved_types)
+            if data_type.reference:
+                data_type.parent = self
+
+        if self.reference:
+            self.reference.children.append(self)
 
     @property
     def type_hint(self) -> str:
         type_: Optional[str] = self.alias or self.type
-        if type_:
-            if (
-                self.reference or self.ref
-            ) and self.python_version == PythonVersion.PY_36:
-                type_ = f"'{type_}'"
-        else:
+        if not type_:
             if len(self.data_types) > 1:
                 type_ = f"Union[{', '.join(data_type.type_hint for data_type in self.data_types)}]"
             elif len(self.data_types) == 1:
@@ -135,9 +169,14 @@ class DataType(BaseModel):
                     f"Literal[{', '.join(repr(literal) for literal in self.literals)}]"
                 )
             else:
-                # TODO support strict Any
-                # type_ = 'Any'
-                type_ = ''
+                if self.reference:
+                    type_ = self.reference.short_name
+                else:
+                    # TODO support strict Any
+                    # type_ = 'Any'
+                    type_ = ''
+        if self.reference and self.python_version == PythonVersion.PY_36:
+            type_ = f"'{type_}'"
         if self.is_list:
             if self.use_generic_container:
                 list_ = 'Sequence'
@@ -153,7 +192,7 @@ class DataType(BaseModel):
                 dict_ = 'dict'
             else:
                 dict_ = 'Dict'
-            type_ = f'{dict_}[str, {type_}]' if type_ else 'dict_'
+            type_ = f'{dict_}[str, {type_}]' if type_ else dict_
         if self.is_optional and type_ != 'Any':
             type_ = f'Optional[{type_}]'
         elif self.is_func:
