@@ -66,7 +66,7 @@ def test_get_model_by_path(schema: Dict, path: str, model: Dict):
 def test_json_schema_object_ref_url_json(mocker):
     parser = JsonSchemaParser('')
     obj = JsonSchemaObject.parse_obj(
-        {'$ref': 'https://example.org/person.schema.json#/definitions/User'}
+        {'$ref': 'https://example.com/person.schema.json#/definitions/User'}
     )
     mock_get = mocker.patch('httpx.get')
     mock_get.return_value.text = json.dumps(
@@ -87,10 +87,7 @@ def test_json_schema_object_ref_url_json(mocker):
     )
     parser.parse_ref(obj, ['Model'])
     mock_get.assert_has_calls(
-        [
-            call('https://example.org/person.schema.json'),
-            call('https://example.com/person.schema.json'),
-        ]
+        [call('https://example.com/person.schema.json'),]
     )
 
 
@@ -366,14 +363,14 @@ def test_parse_nested_array():
 )
 def test_get_data_type(schema_type, schema_format, result_type, from_, import_):
     if from_ and import_:
-        imports_: Optional[List[Import]] = [Import(from_=from_, import_=import_)]
+        imports: Optional[List[Import]] = [Import(from_=from_, import_=import_)]
     else:
-        imports_ = []
+        imports = []
 
     parser = JsonSchemaParser('')
     assert parser.get_data_type(
         JsonSchemaObject(type=schema_type, format=schema_format)
-    ) == DataType(type=result_type, imports_=imports_)
+    ) == DataType(type=result_type, imports=imports)
 
 
 @pytest.mark.parametrize(
@@ -385,5 +382,5 @@ def test_get_data_type_array(schema_types, result_types):
     assert parser.get_data_type(JsonSchemaObject(type=schema_types)) == DataType(
         data_types=[DataType(type=r,) for r in result_types],
         is_optional='null' in schema_types,
-        imports_=[IMPORT_OPTIONAL] if 'null' in schema_types else [],
+        imports=[IMPORT_OPTIONAL] if 'null' in schema_types else [],
     )
