@@ -306,10 +306,7 @@ class Results(BaseModel):
 
 
 def test_openapi_parser_parse_enum_models():
-    parser = OpenAPIParser(
-        Path(DATA_PATH / 'enum_models.yaml').read_text(),
-        data_model_field_type=DataModelFieldBase,
-    )
+    parser = OpenAPIParser(Path(DATA_PATH / 'enum_models.yaml').read_text(),)
     expected_dir = EXPECTED_OPEN_API_PATH / 'openapi_parser_parse_enum_models'
     assert parser.parse() == (expected_dir / 'output_py37.py').read_text()
 
@@ -340,6 +337,26 @@ def test_openapi_parser_parse_nested_anyof():
     )
 
 
+def test_openapi_parser_parse_oneof():
+    parser = OpenAPIParser(Path(DATA_PATH / 'oneof.yaml'),)
+    assert (
+        parser.parse()
+        == (
+            EXPECTED_OPEN_API_PATH / 'openapi_parser_parse_oneof' / 'output.py'
+        ).read_text()
+    )
+
+
+def test_openapi_parser_parse_nested_oneof():
+    parser = OpenAPIParser(Path(DATA_PATH / 'nested_oneof.yaml').read_text(),)
+    assert (
+        parser.parse()
+        == (
+            EXPECTED_OPEN_API_PATH / 'openapi_parser_parse_nested_oneof' / 'output.py'
+        ).read_text()
+    )
+
+
 def test_openapi_parser_parse_allof():
     parser = OpenAPIParser(Path(DATA_PATH / 'allof.yaml'),)
     assert (
